/*
 * aodv.genericList.c
 *
 *  Created on: 10.07.2009
 *      Author: Janos
 */

#include "aodv.genericList.h"

const uint8_t SIZEOF_TYPE = sizeof(aodv_list_t);

void* list_createEntry(aodv_list_t **aList, uint8_t dataSize) {

	aodv_list_t *newEntry = malloc(SIZEOF_TYPE);
	if (newEntry == NULL || aList == NULL) {
		return NULL;
	}
	void *data_ptr = malloc(dataSize);
	if (data_ptr == NULL) {
		free(newEntry);
		return NULL;
	}

	memset(newEntry, 0, SIZEOF_TYPE);
	memset(data_ptr, 0, dataSize);

	if (*aList != NULL) {
		newEntry->next = *aList;
		(*aList)->prev = newEntry;
	}
	*aList = newEntry;
	newEntry->data = data_ptr;

	return data_ptr;
}

aodv_list_t *list_createCustomEntry(aodv_list_t **aList, void *data_ptr) {
	aodv_list_t *newEntry = malloc(SIZEOF_TYPE);
	if (newEntry == NULL || aList == NULL) {
		return NULL;
	}
	memset(newEntry, 0, SIZEOF_TYPE);

	if (*aList != NULL) {
		newEntry->next = *aList;
		(*aList)->prev = newEntry;
	}
	*aList = newEntry;
	newEntry->data = data_ptr;
	return newEntry;
}

bool list_deleteEntry(aodv_list_t **aList, void *anEntry) {
	aodv_list_t *listElement;
	if (anEntry == NULL || aList == NULL) {
		return false;
	}
	listElement = list_findEntry(*aList, anEntry);
	if (listElement == NULL) {
		return false;
	}
	if (listElement->prev != NULL) {
		listElement->prev->next = listElement->next;
	}
	else {
		*aList = listElement->next;
	}
	if (listElement->next != NULL) {
		listElement->next->prev = listElement->prev;
	}
	free(listElement->data);
	free(listElement);

	return true;
}
aodv_list_t* list_findEntry(aodv_list_t *aList, void *anEntry) {
	while (aList != NULL) {
		if (aList->data == anEntry) {
			return aList;
		}
		aList = aList->next;
	}
	return NULL;
}
void list_clearList(aodv_list_t **aList) {
	aodv_list_t *next;
	aodv_list_t *tmp;
	next = *aList;
	while (next != NULL) {
		tmp = next->next;
		free(next->data);
		free(next);
		next = tmp;
	}
	*aList = NULL;
}

uint8_t list_getSize(aodv_list_t *aList) {
	uint8_t count = 0;
	while (aList != NULL) {
		count++;
		aList = aList->next;
	}
	return count;
}
