/*
 * aodv.interface.c
 *
 *  Created on: 20.05.2009
 *      Author: Janos
 */

#include "aodv.interface.h"
#include "aodv.routing.h"

#define LOGTAG 		("interface")

/* List<dataReceivedListener> */
aodv_list_t *onPacketReceivedListeners = NULL;
/* List<RREPReceivedListener> */
aodv_list_t *onRREPReceivedListeners = NULL;
/* List<RREQReceivedListener> */
aodv_list_t *onRREQReceivedListeners = NULL;
/* List<dataReceivedListener> */
aodv_list_t *onPacketForwardListeners = NULL;

aodv_dataPacket_t TRANSMIT_DATA_PACKET;

void aodv_interface_initAODV(void) {
	startSendHELLO_timer();
}

bool aodv_interface_addDataReceivedListener(dataReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onPacketReceivedListeners, aListener);
	if (entry == NULL) {
		entry = list_createCustomEntry(&onPacketReceivedListeners, aListener);
		return entry != NULL;
	}
	return false;
}
bool aodv_interface_removeDataReceivedListener(dataReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onPacketReceivedListeners, aListener);
	if (entry != NULL) {
		return list_deleteEntry(&onPacketReceivedListeners, entry);
	}
	return false;
}
void aodv_interface_invokeDataReceivedListeners(aodv_dataPacket_t *receivedData) {
	aodv_list_t *tmp = onPacketReceivedListeners;
	while (tmp != NULL) {
		((dataReceivedListener) tmp->data)(receivedData);
		tmp = tmp->next;
	}
}

bool aodv_interface_addRREPReceivedListener(RREPReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onRREPReceivedListeners, aListener);
	if (entry == NULL) {
		entry = list_createCustomEntry(&onRREPReceivedListeners, aListener);
		return entry != NULL;
	}
	return false;
}
bool aodv_interface_removeRREPReceivedListener(RREPReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onRREPReceivedListeners, aListener);
	if (entry != NULL) {
		return list_deleteEntry(&onRREPReceivedListeners, entry);
	}
	return false;
}
void aodv_interface_invokeRREPReceivedListeners(aodv_RREP_message_t *receivedRREP) {
	aodv_list_t *tmp = onRREPReceivedListeners;
	while (tmp != NULL) {
		((RREPReceivedListener) tmp->data)(receivedRREP);
		tmp = tmp->next;
	}
}

bool aodv_interface_addRREQReceivedListener(RREQReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onRREQReceivedListeners, aListener);
	if (entry == NULL) {
		entry = list_createCustomEntry(&onRREQReceivedListeners, aListener);
		return entry != NULL;
	}
	return false;
}
bool aodv_interface_removeRREQReceivedListener(RREQReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onRREQReceivedListeners, aListener);
	if (entry != NULL) {
		return list_deleteEntry(&onRREQReceivedListeners, entry);
	}
	return false;
}
void aodv_interface_invokeRREQReceivedListeners(aodv_RREQ_message_t *receivedRREQ) {
	aodv_list_t *tmp = onRREQReceivedListeners;
	while (tmp != NULL) {
		((RREQReceivedListener) tmp->data)(receivedRREQ);
		tmp = tmp->next;
	}
}

bool aodv_interface_addPacketForwardListener(dataReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onPacketForwardListeners, aListener);
	if (entry == NULL) {
		entry = list_createCustomEntry(&onPacketForwardListeners, aListener);
		return entry != NULL;
	}
	return false;
}
bool aodv_interface_removePacketForwardListener(dataReceivedListener aListener) {
	aodv_list_t *entry = list_findEntry(onPacketForwardListeners, aListener);
	if (entry != NULL) {
		return list_deleteEntry(&onPacketForwardListeners, entry);
	}
	return false;
}
void aodv_interface_invokePacketForwardListeners(aodv_dataPacket_t *receivedData) {
	aodv_list_t *tmp = onPacketForwardListeners;
	while (tmp != NULL) {
		((dataReceivedListener) tmp->data)(receivedData);
		tmp = tmp->next;
	}
}


void aodv_interface_sendData(netaddr_t destination, const void *data, uint16_t dataSize, uint8_t flags, uint16_t application, aodv_applicationCallback callback_handler) {
	TRANSMIT_DATA_PACKET.header.destination = destination;
	TRANSMIT_DATA_PACKET.header.originator = Configuration.id;
	TRANSMIT_DATA_PACKET.header.flags = 0;
	TRANSMIT_DATA_PACKET.header.ttl = NET_DIAMETER;
	TRANSMIT_DATA_PACKET.header.application = application;
	TRANSMIT_DATA_PACKET.payload_size = dataSize;
	TRANSMIT_DATA_PACKET.payload = data;

	log(LOGTAG, "Sending data to '%i'.", destination);

	aodvAttemptRouteDataPacket(&TRANSMIT_DATA_PACKET, flags, callback_handler);

	logsep();
}

bool aodv_interface_sendDataFrame(aodv_dataPacket_t *aDataPacket, netaddr_t destination) {

	netpacket_send_args_t npsargs;
	void *tmpDataBuffer;
	bool bSuccess = false;
	uint8_t headerSize = sizeof(aodv_header_t);
	uint16_t packetSize = headerSize + aDataPacket->payload_size;

	log(LOGTAG, "Sending data Frame to '%i'.", destination);

	tmpDataBuffer = malloc(packetSize);
	if (tmpDataBuffer == NULL) {
		log(LOGTAG, "FATAL: No memory available for buffering the data!");
		logsep();
		return false;
	}

	memcpy(tmpDataBuffer, &aDataPacket->header, headerSize);
	memcpy(tmpDataBuffer + headerSize, aDataPacket->payload, aDataPacket->payload_size);

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = destination;
	npsargs.netheader.type = AODV_DATA_PACKAGE;
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = tmpDataBuffer;
	npsargs.payload[0].size = packetSize;

	logsep();

	bSuccess = Net_send(&npsargs);

	free(tmpDataBuffer);

	return bSuccess;
}

bool sendPlainData(netaddr_t dest, void *data, size_t size) {
	netpacket_send_args_t npsargs;
	Net_sendArgsInit(&npsargs);
	npsargs.netheader.to = dest;
	npsargs.netheader.type = AODV_TEST_PACKAGE;
	npsargs.netheader.flags = 0;
	npsargs.payload[0].buffer = data;
	npsargs.payload[0].size = size;
	return Net_send(&npsargs);
}

bool aodv_interface_sendRREPPackage(aodv_RREP_message_t *aRouteReply, netaddr_t destination) {

	netpacket_send_args_t npsargs;

	log(LOGTAG, "Sending RREP to '%i' over nexthop '%i'.", aRouteReply->origIPAddress, destination);

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = destination;				// Destination: nextHop
	npsargs.netheader.type = AODV_CONTROL_MESSAGE;	// AODV control message type
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = aRouteReply;
	npsargs.payload[0].size = sizeof(aodv_RREQ_message_t);

	return Net_send(&npsargs);
}

bool aodv_interface_broadcastRREQPackage(aodv_RREQ_message_t *aRouteRequest) {

	netpacket_send_args_t npsargs;

	log(LOGTAG, "Broadcasting RREQ for '%i' (RREQ-ID=%i).", aRouteRequest->destIPAddress, aRouteRequest->requestID);

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = NETADDR_BROADCAST;		// Destination: Broadcast
	npsargs.netheader.type = AODV_CONTROL_MESSAGE;	// AODV control message type
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = aRouteRequest;
	npsargs.payload[0].size = sizeof(aodv_RREQ_message_t);

	return Net_send(&npsargs);
}
