/*
 * aodv.buffers.c
 *
 *  Created on: 21.05.2009
 *      Author: Janos
 */

#include "aodv.packetbuffer.h"

#define LOGTAG 		("packetbuffer")

aodv_list_t *PACKET_BUFFER = NULL;
uint8_t numBufferEntries = 0;

inline void preparePacketBufferEntry(aodv_packetBuffer_entry_t* bufferEntry) {
	/* Erase memory. */
	memset(bufferEntry, 0x0, sizeof(aodv_packetBuffer_entry_t));
	bufferEntry->nextTTL = TTL_START;
	bufferEntry->nextBackoffTime = RING_TRAVERSAL_TIME(TTL_START);

}

aodv_packetBuffer_entry_t* aodv_bufferDataPacket(aodv_dataPacket_t *anAodvDataPacket,
		aodv_applicationCallback callback_handler, uint8_t rreq_flags) {

	aodv_packetBuffer_entry_t *newBufferEntry;
	void *payload;

	log(LOGTAG, "Buffering packet in PACKET_BUFFER.");

	newBufferEntry = list_createEntry(&PACKET_BUFFER, sizeof(aodv_packetBuffer_entry_t));

	if (newBufferEntry == NULL) {
		/**
		 * No PACKET_BUFFER entry available!
		 */
		log(LOGTAG, "No memory for payload of size %i (bytes) available!", anAodvDataPacket->payload_size);
		return NULL;
	}

	payload = malloc(anAodvDataPacket->payload_size);
	if (payload == NULL) {
		log(LOGTAG, "No memory for payload of size %i (bytes) available!", anAodvDataPacket->payload_size);
		list_deleteEntry(&PACKET_BUFFER, newBufferEntry);
		return NULL;
	}

	numBufferEntries++;

	preparePacketBufferEntry(newBufferEntry);

	/* Store the packets payload in the buffer. */
	memcpy(payload, anAodvDataPacket->payload, anAodvDataPacket->payload_size);
	newBufferEntry->bufferedData = payload;
	newBufferEntry->data_size = anAodvDataPacket->payload_size;

	/* Store the packets header in the buffer. */
	memcpy(&newBufferEntry->packetHeader, &anAodvDataPacket->header, sizeof(aodv_header_t));

	/* Mark the slot as "occupied". */
	newBufferEntry->callback_handler = callback_handler;
	newBufferEntry->rreq_flags = rreq_flags;

	log(LOGTAG, "Packet buffered (#%i).", numBufferEntries);

	return newBufferEntry;
}

void aodv_dropBufferedDataPacket(aodv_packetBuffer_entry_t *bufferEntry) {
	log(LOGTAG, "FREEing buffered packet data for destination %i (%i packets remaining).", bufferEntry->packetHeader.destination, --numBufferEntries);
	free(bufferEntry->bufferedData);
	list_deleteEntry(&PACKET_BUFFER, bufferEntry);
}
