/*
 * aodv.packetbuffer.h
 *
 *  Created on: 21.05.2009
 *      Author: Janos
 */

#ifndef AODV_DATABUFFER_H_
#define AODV_DATABUFFER_H_

#include <string.h>

#include <ScatterWeb.System.h>
#include <ScatterWeb.Net.h>

#include "aodv.interface.h"
#include "aodv.genericList.h"

extern aodv_list_t *PACKET_BUFFER;

/**
 * Struct for buffered data packets.
 * This struct represents an entry for one packet
 * in the packet buffer.
 * The entry stores the original AODV-header, the payload and
 * the payloads size.
 */
typedef struct aodv_packetBuffer_entry_s aodv_packetBuffer_entry_t;
struct aodv_packetBuffer_entry_s {

	void *bufferedData;							// Pointer to the buffered payload data. Memory has to be allocated.
	uint16_t data_size; 						// Size of buffered data.
	aodv_header_t packetHeader; 				// Header of buffered data.

	aodv_applicationCallback callback_handler;	// Application callback handler pointer. (may be NULL)
	uint8_t rreq_flags;

	uint8_t nextTTL;							// TTL value for the next retry.
	uint8_t numRetries;							// Number of retries for this buffered packet.
	uint16_t nextBackoffTime;					// Time (in ms) for the next retry for sending this data.
};

/**
 * @brief Buffers an AODV-data-packet which is awaitening a route to it's
 * destination to be established.
 *
 * The packet will be buffered for unpredictable time in a ringbuffer
 * and will be overwritten, if the buffer overloads.
 *
 * @param anAodvDataPacket The data packet which is stored with the purpose to
 * send it again when a route to it's destination is established.
 * @param callback_handler The application callback handler which is used to
 * inform the application over the success or failure of the sending of the
 * data packet.
 *
 * @return The application callback handler for the data buffer entry which might
 * have been overwritten by storing the current data. This handler can be used
 * by the caller to inform the application that a data packet couldn't be sent.
 */
aodv_packetBuffer_entry_t* aodv_bufferDataPacket(
		aodv_dataPacket_t *anAodvDataPacket,
		aodv_applicationCallback callback_handler,
		uint8_t rreq_flags
);

void aodv_dropBufferedDataPacket(aodv_packetBuffer_entry_t *aDataBufferEntry);

#endif /* AODV_DATABUFFER_H_ */
