/*
 * aodv.routing.h
 *
 *  Created on: 13.05.2009
 *      Author: Janos
 */

#ifndef AODV_ROUTING_H_
#define AODV_ROUTING_H_

#include <string.h>

#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <ScatterWeb.Net.h>
#include <ScatterWeb.Timers.h>

#include "log.h"
#include "aodv.test.configuration.h"

#include "controller.basicIO.h"

#include "aodv.interface.h"
#include "aodv.routing.table.h"
#include "aodv.messageTypes.h"
#include "aodv.packetbuffer.h"
#include "aodv.routing.rreq.h"
#include "aodv.routing.rerr.h"




/**
 * @brief Stops all pending timers for a HELLO broadcast and starts a new one.
 *
 * This starts the delayed broadcast of a HELLO message after HELLO_INTERVAL
 * milliseconds. Before doing this, all currently pending delayed broadcasts
 * are canceled.
 *
 * This method should always be called after an AODV message is broadcasted.
 */
void startSendHELLO_timer(void);

/*
 * @brief Processes an AODV control message.
 *
 * AODV control messages such es RREQ, RREQ, RERR and RREP-ACK messages
 * are processed in this method.
 *
 * The AODV-nature of the IP package is determined by the
 * AODV_CONTROL_MESSAGE-type in the IP header.
 *
 * Control messages are either Route Requests (RREQ), Route Replies (RREP),
 * Route Errors (RERR) or Route Reply Acknowledgments (RREP-ACK).
 * If this node is aimed at in one of such messages, it will fulfill
 * its duty in participating in an AODV-driven mobile Ad-hoc-network
 * and forward or reply messages.
 *
 * However, control messages are, independently from their intention,
 * most likely used to update the AODV protocol state information.
 * This means, that also AODV control messages will be examined and
 * processed even when this node doesn't have anything to do with
 * it.
 *
 * @param args The "IP"-Package that was captured by this node.
 */
void aodvProcessControlMessage(struct netpacket_handler_args* aNetPacket);

/*
 * @brief Handles a data packet that was sent to >this< node.
 *
 * The data packet was sent to >this< node, but contains in its
 * payload the designated destination. If >this< node is not the
 * actual destination of the package, the package must be routed
 * towards the destination using the AODV protocol.
 *
 * If the AODV routing protcol does not know a path to the
 * destination of the package, all necessary actions to achieve
 * this informations are taken.
 * The AODV protocol will forward the package towards the next-hop
 * node, after all necessary information is available.
 *
 * @param args The layer-2 packet that was captured by this node.
 */
void aodvProcessDataPacket(struct netpacket_handler_args* args);

/**
 * @brief Interface for sending data to a destination using the
 * AODV routing facilities.
 *
 * Attempts to route a data packet to a destination which is not
 * the current node. This fact MUST have been meade shure prior to
 * the call of this method. Otherwise this method will either route
 * the packet towards itself or buffer it and send a request to itself!
 *
 * The AODV protocol will look for a route to the desired destination
 * in the routing table and forward it, if an entry is found.
 * If not, the packet will be buffered in the PACKET_BUFFER and
 * all necessary actions to establish a route will be taken. That is,
 * sending a RREQ towards the desired destination.
 *
 * The return value is TRUE, if a route could be found and the packet
 * was sent, and FALSE, if the packet had to be buffered.
 */
bool aodvAttemptRouteDataPacket(aodv_dataPacket_t *anAodvDataPacket, uint8_t flags, aodv_applicationCallback handler);

#endif /* AODV_ROUTING_H_ */
