/*
 * aodv.routing.rerr.c
 *
 *  Created on: 14.07.2009
 *      Author: Janos
 */
#include "aodv.routing.rerr.h"

/*
 * Receiving RERRs:
 * - Procedure#1:
 *   - Make LIST:
 *   	- Add all destination from the RERR, for which a routing table
 *   	  entry exists, which has the SENDER of the RERR as NEXTHOP.
 *   - Update Routing table
 *   	- For every destination in the RERR
 *   		- If routing table exists for that destination
 *   			- Copy the destination sequence number from the RERR into the entry
 *   			- Invalidate the entry
 *   			- Start deletion after DELETE_PERIOD
 *   - Procedure#3(LIST). (Prepare RERR).
 *   - Procedure#4.
 */
aodv_list_t *aodv_makeListFromRERR(struct netpacket_handler_args* aNetPacket, aodv_RERR_message_t *aRERR) {

	uint8_t destCount = aRERR->destCount;
	uint8_t i;

	netaddr_t sender = aNetPacket->netheader->from;

	netaddr_t *destinations = aRERR->unreachableDestIPAddresses;
	uint16_t *sequenceNumbers = aRERR->unreachableDestSeqNumbers;

	aodv_routingTableEntry_t *tmpEntry;

	aodv_list_t *findEntry;
	aodv_list_t *list = NULL;

	for (i = 0; i < destCount; i++) {
		tmpEntry = aodv_routingTable_lookupRoute(destinations[i]);
		if (tmpEntry != NULL) {
			if (tmpEntry->nextHopAddress == sender) {
				findEntry = list_findEntry(list, tmpEntry);
				if (findEntry == NULL) {
					list_createCustomEntry(&list, tmpEntry);
				}
			}
			tmpEntry->valid = false;
			tmpEntry->destSeqNumber = sequenceNumbers[i];
			// TODO: Start deletion timer for that route.
		}
	}
	return list;
}
