/*
 * aodv.routing.rrep.c
 *
 *  Created on: 03.06.2009
 *      Author: Janos
 */

#include "aodv.routing.rrep.h"

#define LOGTAG 		("RREP")

void aodv_processHELLO(struct netpacket_handler_args* aNetPacket,
		aodv_RREP_message_t *preparedRREP) {

	aodv_routingTableEntry_t* forwardRoute;				// The route for the destination (the originator of the RREP)

	memcpy(preparedRREP, aNetPacket->payload, sizeof(aodv_RREP_message_t));

	log(LOGTAG, "Processing HELLO from %i.", preparedRREP->destIPAddress);

	log(LOGTAG, "Updating forward route to destination '%i' of RREP.", preparedRREP->destIPAddress);
	forwardRoute = aodv_routingTable_updateForwardRouteFromRREP(
			preparedRREP, aNetPacket->netheader->from);


}

checkRREPResult_t aodv_processRREP(struct netpacket_handler_args* aNetPacket,
		aodv_RREP_message_t *preparedRREP) {

	/**
	 * Attention:
	 * Mind the special naming of the route variable names!
	 */
	aodv_routingTableEntry_t* reverseRoute;				// The route to which the RREP is FORWARDED.
	aodv_routingTableEntry_t* forwardRoute;				// The route for the destination (the originator of the RREP)
	aodv_routingTableEntry_t* previousHopRoute;			// The route for the node, from which the RREP was received.

	checkRREPResult_t processingResult;

	memcpy(preparedRREP, aNetPacket->payload, sizeof(aodv_RREP_message_t));

	log(LOGTAG, "Processing RREP from %i to %i.", preparedRREP->destIPAddress, preparedRREP->origIPAddress);

	/* Discard RREP if it was originated by >this< node. */
	if (preparedRREP->destIPAddress == Configuration.id) {
		log(LOGTAG, "RREP was originated from >this< node (%i)", Configuration.id);
		logm("--> RREP was dropped.\r\n");
		processingResult.result = RREP_SELF_ORIGINATED;
		return processingResult;
	}


	/* Create/Update the route to the previous hop, using the IP-Packet. */
	log(LOGTAG, "Updating route to previousHop '%i'.", aNetPacket->netheader->from);
	previousHopRoute = aodv_routingTable_updateEntryFromIP(aNetPacket->netheader->from);

	/*
	 * Create or update forward route to the DEST_IP and update the RREP for forwarding.
	 */
	log(LOGTAG, "Updating forward route to destination '%i' of RREP.", preparedRREP->destIPAddress);
	forwardRoute = aodv_routingTable_updateForwardRouteFromRREP(
			preparedRREP, aNetPacket->netheader->from);

	if (preparedRREP->origIPAddress == Configuration.id) {
		/**
		 * RREP has reached its destination (>this< node).
		 */
		log(LOGTAG, "RREP has reached >THIS< node.\r\n");
		processingResult.result = RREP_REACHED_DESTINATION;
		return processingResult;
	}

	reverseRoute = aodv_routingTable_lookupRoute(preparedRREP->origIPAddress);
	if (reverseRoute == NULL) {
		log(LOGTAG, "RREP cannot be forwarded! No route towards RREPs origination '%i' found.", preparedRREP->origIPAddress);
		processingResult.result = RREP_NOROUTE;
		return processingResult;
	}

	if (previousHopRoute != NULL) {
		aodv_routingTable_addPrecursor(previousHopRoute, reverseRoute->nextHopAddress);
	}
	if (forwardRoute != NULL) {
		aodv_routingTable_addPrecursor(forwardRoute, reverseRoute->nextHopAddress);
	}

	processingResult.result = RREP_FORWARD;
	processingResult.reverseRoute = reverseRoute;

	return processingResult;
}

void prepareIntermediateRREPMessageFromRREQ(aodv_RREQ_message_t *aRREQ,
		aodv_RREP_message_t *aRREP, aodv_routingTableEntry_t *forwardRoute) {

	memset(aRREP, 0, sizeof(aodv_RREP_message_t));

	aRREP->type = TYPE_RREP;
	aRREP->hopCount = forwardRoute->hopCount;
	aRREP->origIPAddress = aRREQ->origIPAddress;
	aRREP->destIPAddress = aRREQ->destIPAddress;
	aRREP->destSeqNum = forwardRoute->destSeqNumber;
}

void prepareGratuitousRREPMessageFromRREQ(aodv_RREQ_message_t *aRREQ,
		aodv_RREP_message_t *aRREP, aodv_routingTableEntry_t *reverseRoute) {

	memset(aRREP, 0, sizeof(aodv_RREP_message_t));

	aRREP->type = TYPE_RREP;
	aRREP->hopCount = reverseRoute->hopCount;
	aRREP->origIPAddress = aRREQ->destIPAddress;
	aRREP->destIPAddress = aRREQ->origIPAddress;
	aRREP->destSeqNum = aRREQ->origSeqNum;
}

void prepareRREPMessageFromRREQ(aodv_RREQ_message_t *aRREQ,
		aodv_RREP_message_t *aRREP) {

	uint16_t ownDestSeqNum = getOwnDestinationSequenceNumber();

	if (!aRREQ->unknownSeqNum && aRREQ->destSeqNum == ownDestSeqNum + 1) {
		ownDestSeqNum = increaseOwnDestinationSequenceNumber();
	}

	memset(aRREP, 0, sizeof(aodv_RREP_message_t));

	aRREP->type = TYPE_RREP;
	aRREP->destIPAddress = aRREQ->destIPAddress;
	aRREP->destSeqNum = aRREQ->origSeqNum;
	aRREP->origIPAddress = aRREQ->origIPAddress;
	aRREP->hopCount = 0;

}
