/*
 * aodv.routing.rreq.c
 *
 *  Created on: 28.05.2009
 *      Author: Janos
 */

#include "aodv.routing.rreq.h"

#define LOGTAG 		("RREQ")

/**
 * List of type <rreqBufferEntry_t>.
 */
aodv_list_t *rreqBuffer_list = NULL;
uint8_t SIZEOF_RREQBUFFERENTRY_T = sizeof(rreqBufferEntry_t);


inline bool checkRREQBuffer(aodv_RREQ_message_t *aRREQ) {
	aodv_list_t *tmp = rreqBuffer_list;
	rreqBufferEntry_t *entry;
	while (tmp != NULL) {
		entry = tmp->data;
		if (entry->origIPAddress == aRREQ->origIPAddress && entry->requestID == aRREQ->requestID) {
			return true;
		}
		tmp = tmp->next;
	}
	return false;
}

void bufferRREQ(aodv_RREQ_message_t *aRREQ) {

	rreqBufferEntry_t *newBufferEntry;

	log(LOGTAG, "Buffering RREQ, origIP: %i, RREQ-ID: %i.", aRREQ->origIPAddress, aRREQ->requestID);

	newBufferEntry = list_createEntry(&rreqBuffer_list, SIZEOF_RREQBUFFERENTRY_T);

	if (newBufferEntry == NULL) {
		log(LOGTAG, "FATAL: A RREQ from the destination %i with RREQ-ID %i could not be buffered!", aRREQ->origIPAddress, aRREQ->requestID);
		return;
	}

	newBufferEntry->origIPAddress = aRREQ->origIPAddress;
	newBufferEntry->requestID = aRREQ->requestID;

}

checkRREQResult_t aodv_processRREQ(struct netpacket_handler_args* aNetPacket, aodv_RREQ_message_t *preparedRREQ) {

	aodv_routingTableEntry_t* forwardRoute;		// route towards the destination of the RREQ.
	aodv_routingTableEntry_t* reverseRoute;		// route towards the originator of the RREQ.
	aodv_routingTableEntry_t* nextHopRoute;		// route of the next hop towards the destination.
												// Needed for adding the previous hop as a precursor, if
												// a forward route exists.
	int16_t compareResult;
	checkRREQResult_t processingResult;

	memcpy(preparedRREQ, aNetPacket->payload, sizeof(aodv_RREQ_message_t));

	log(LOGTAG, "Route Requested: {RREQ_ID: %i, to: %i, orig: %i, prevHop: %i}",
			preparedRREQ->requestID, preparedRREQ->destIPAddress, preparedRREQ->origIPAddress, aNetPacket->netheader->from);

	/* Discard RREQ if it was originated by >this< node. */
	if (preparedRREQ->origIPAddress == Configuration.id) {
		log(LOGTAG, "RREQ was originated from >this< node (%i)", Configuration.id);
		logm("--> RREQ was dropped.");
		processingResult.result = RREQ_SELF_ORIGINATED;
		return processingResult;
	}

	/* Create/Update the route to the previous hop, using the IP-Packet. */
	log(LOGTAG, "Updating route to previousHop '%i'.", aNetPacket->netheader->from);
	aodv_routingTable_updateEntryFromIP(aNetPacket->netheader->from);

	/*
	 * Check the RREQ-Buffer for this RREQ and discard it, if it has already been
	 * received earlier.
	 */
	log(LOGTAG, "Checking RREQ-Buffer for stored RREQs...");
	if (checkRREQBuffer(preparedRREQ)) {
		/* RREQ already received earlier --> discard it. */
		log(LOGTAG, "RREQ from '%i' with RREQ-ID '%i' already received. Discarding this RREQ.", preparedRREQ->origIPAddress, preparedRREQ->requestID);
		processingResult.result = RREQ_DUPLICATE;
		return processingResult;
	}
	log(LOGTAG, "RREQ not found in RREQ-Buffer. Processing it...");
	bufferRREQ(preparedRREQ);

	/*
	 * Create or update reverse route to the ORIG_IP and update the RREQ for re-broadcasting.
	 */
	log(LOGTAG, "Updating reverse route to originator of RREQ '%i'.", preparedRREQ->origIPAddress);
	reverseRoute = aodv_routingTable_updateReverseRouteFromRREQ(preparedRREQ, aNetPacket->netheader->from);

	if (preparedRREQ->destIPAddress == Configuration.id) {
		log(LOGTAG, "RREQ has reached this node.");
		processingResult.result = RREQ_REACHED_DESTINATION;
		processingResult.reverseRoute = reverseRoute;
		return processingResult;
	}

	/* Search for a route towards the destination of the RREQ. */
	forwardRoute = aodv_routingTable_lookupRoute(preparedRREQ->destIPAddress);
	processingResult.forwardRoute = forwardRoute;

	if (forwardRoute != NULL) {
		/* Route to destination exists */

		nextHopRoute = aodv_routingTable_lookupRoute(forwardRoute->nextHopAddress);
		if (nextHopRoute != NULL) {
			// Adding the PREV_HOP into the precursor list of the NEXT_HOP route.
			aodv_routingTable_addPrecursor(nextHopRoute, aNetPacket->netheader->from);
		}
		// Adding the PREV_HOP into the precursor list of the forward route.
		aodv_routingTable_addPrecursor(forwardRoute, aNetPacket->netheader->from);

		if (!preparedRREQ->destOnly) {
			/* RREP by this node as an INTERMEDIATE node can be sent. */

			processingResult.reverseRoute = reverseRoute;

			if (preparedRREQ->gratuitous) {
				log(LOGTAG, "RREQ is GRATUITOUS.");
				processingResult.result = RREQ_INTERMEDIATE_GRATUITOUS_RREP;
			}
			else {
				processingResult.result = RREQ_INTERMEDIATE_RREP;
			}
			return processingResult;
		}
		else {
			log(LOGTAG, "RREQ is DESTONLY.");
			/* Update the RREQ destSeqNum if possible. */
			if (!preparedRREQ->unknownSeqNum) {
				compareResult = ((signed) forwardRoute->destSeqNumber) - ((signed) preparedRREQ->destSeqNum);
				if (compareResult > 0) {
					preparedRREQ->destSeqNum = forwardRoute->destSeqNumber;
				}
			}
			else if (forwardRoute->validDestSeqNumber) {
				preparedRREQ->destSeqNum = forwardRoute->destSeqNumber;
			}
		}
	}

	processingResult.result = RREQ_REBROADCAST;
	return processingResult;
}

void prepareRREQMessage(aodv_RREQ_message_t *aRREQMessage, aodv_packetBuffer_entry_t *bufferEntry, aodv_routingTableEntry_t *aDestRoutingTableEntry) {

	aRREQMessage->type = AODV_MESSAGE_TYPE_RREQ;
	aRREQMessage->destIPAddress = bufferEntry->packetHeader.destination;
	if (aDestRoutingTableEntry) {
		aRREQMessage->destSeqNum = aDestRoutingTableEntry->destSeqNumber;
		aRREQMessage->unknownSeqNum = false;
	}
	else {
		aRREQMessage->destSeqNum = 0;
		aRREQMessage->unknownSeqNum = true;
	}
	aRREQMessage->hopCount = 0;
	aRREQMessage->destOnly = ((bufferEntry->rreq_flags & AODV_FLAG_DESTONLY) ? true : false);
	aRREQMessage->gratuitous = ((bufferEntry->rreq_flags & AODV_FLAG_GRATUITOUS) ? true : false);
	aRREQMessage->join = false;
	aRREQMessage->repair = false;
	aRREQMessage->origIPAddress = Configuration.id;
	aRREQMessage->origSeqNum = getOwnDestinationSequenceNumber();
	aRREQMessage->requestID = getNextRequestID();
	aRREQMessage->reserved = 0;
	aRREQMessage->ttl = bufferEntry->nextTTL;

}
