/*
 * aodv.routing.rreq.h
 *
 *  Created on: 28.05.2009
 *      Author: Janos
 */

#ifndef AODV_ROUTING_RREQ_H_
#define AODV_ROUTING_RREQ_H_

#include <string.h>

#include <ScatterWeb.System.h>
#include <ScatterWeb.Net.h>
#include <ScatterWeb.Configuration.h>
#include <ScatterWeb.CC1020.h>

#include "aodv.messageTypes.h"
#include "aodv.configuration.h"
#include "aodv.routing.table.h"
#include "aodv.interface.h"
#include "aodv.routing.rrep.h"
#include "aodv.packetbuffer.h"
#include "aodv.genericList.h"

typedef enum checkRREQResult {
	RREQ_SELF_ORIGINATED,					// RREQ was originated by >this< node.
	RREQ_DUPLICATE,							// RREQ already received and buffered.
	RREQ_REACHED_DESTINATION,				// RREQ has reached it's destination.
	RREQ_REBROADCAST,						// RREQ must be forwarded (rebroadcasted)
	RREQ_INTERMEDIATE_RREP,					// RREP can be sent by >this< intermediate node.
	RREQ_INTERMEDIATE_GRATUITOUS_RREP		// RREP can be sent by >this< intermediate node,
											// but gratuitous RREP must be sent to the destination.
} checkRREQResult_e;

typedef struct checkRREQResult_s {
	checkRREQResult_e result;
	aodv_routingTableEntry_t *reverseRoute;
	aodv_routingTableEntry_t *forwardRoute;
} checkRREQResult_t;

/**
 * Structure for storing information for RREQ-messages which
 * have been received.
 *
 * This structure stores the fields 'origIPAddress' and 'requestID'
 * from the type 'aodv_RREQ_message_t'.
 */
typedef struct rreqBufferEntry_s rreqBufferEntry_t;
struct rreqBufferEntry_s {
	netaddr_t 	origIPAddress;		// Originator of buffered RREQ
	uint16_t 	requestID;			// Request-ID of the buffered RREQ
};

/**
 * FICOMMENTXME: Documentation. This function is important.
 */
checkRREQResult_t aodv_processRREQ(struct netpacket_handler_args* aNetPacket, aodv_RREQ_message_t *preparedRREQ);

/**
 * Prepares a RREQ-message for dissemination.
 * The RREQ will be prepared for finding a route to the destination 'destination'.
 * If available, the last known routing table entry for this destination is supplied.
 */
void prepareRREQMessage(aodv_RREQ_message_t *aRREQMessage, aodv_packetBuffer_entry_t *bufferEntry, aodv_routingTableEntry_t *aDestRoutingTableEntry);


#endif /* AODV_ROUTING_RREQ_H_ */
