/*
 * aodv.test.commands.c
 *
 *  Created on: 14.07.2009
 *      Author: Janos
 */

#include "aodv.tests.h"


void tests_initRandom(void) {

	unsigned int seed;

	System_enableADC();

	seed = System_random();

	srand(seed);
}

uint16_t rand_n(uint16_t max) {
	return rand() % max;
}


uint32_t timer_value = 0;

void incTimer(uint32_t *timer) {
	(*timer) += 2;
	Timers_add(1, (fp_timer) incTimer, timer);
}

uint32_t *startTimer(void) {
	uint32_t *timer = malloc(sizeof(uint32_t));
	if (timer == NULL) {
		return NULL;
	}
	*timer = 0;
	Timers_add(1, (fp_timer) incTimer, timer);
	return timer;
}

uint32_t stopTimer(uint32_t *timer) {
	uint32_t value = 0;
	if (timer != NULL) {
		value = *timer;
		Timers_remove((fp_timer) incTimer, timer);
		free(timer);
	}
	return value;
}

/*****************************************************************************************
 *           COMMAND: Reset Chain                                                        *
 *****************************************************************************************/
bool bResetRequested = false;
void do_reset(void) {
	System_reset(WDS_SWRESET);
}
COMMAND(resetchain, CMDFLAG_NONE, cmdargs) {
	if (!bResetRequested) {
		bResetRequested = true;
		_Cmd_AsciiSerialRequest("@255 resetchain");
	}
	while (Timers_remove((fp_timer) do_reset, NULL));
	Timers_add(1000, (fp_timer) do_reset, (void*) 0xFFFF);

	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "OK");
}
/*****************************************************************************************
 *           END_OF_COMMAND: Reset Chain                                                 *
 *****************************************************************************************/



/*****************************************************************************************
 *           TEST 1: Time of Route Discovery                                             *
 *****************************************************************************************/
#define TEST1_NAME "Time of Route Discovery"
#define TEST1_ID TEST_1_A
#define TEST1_TIMEOUT (5000)
#define TEST1_INTERVAL (500)
uint8_t test1_destination;
uint8_t test1_numberOfTests;
uint8_t test1_numTestsPassed = 0;
uint8_t test1_numSuccesses = 0;

uint32_t test1_sumTimers = 0;
uint32_t test1_minTimer = 0 - 1;
uint32_t test1_maxTimer = 0;

uint16_t test1_sumHopCounts = 0;
uint16_t test1_minHopCount = 0 - 1;
uint16_t test1_maxHopCount = 0;

uint32_t *timer_test1;
uint8_t flags = AODV_FLAG_DESTONLY;
bool bTest1_finished = false;
void test1_initTest(void);
void test1_runTest(void);
void test1_finishTest(void);
void test1_testSuccess(void);
void test1_testFailure(void);
void test1_testTimeout(void);
void test1_restartTest(void);
void onTest1RREPReceived(aodv_RREP_message_t *aRREP);
void onPacketSent(enum aodv_sendPacketStatus_e sendStatus);

void test1_initTest(void) {
	bTest1_finished = false;
	test1_sumTimers = 0;
	test1_numTestsPassed = 0;
	test1_numSuccesses = 0;
	stopTimer(timer_test1);

	// Disable regular logging...
	bDontLog = true;
}
void test1_finishTest(void) {

	aodv_interface_removeRREPReceivedListener(onTest1RREPReceived);
	testfinish(TEST1_ID, TEST1_NAME);
	testlogm("%i of %i measures were taken.", test1_numSuccesses, test1_numberOfTests);
	testlogm("Average discovery time: %10lu/%i = %10lu ms.", test1_sumTimers, test1_numSuccesses, test1_sumTimers / test1_numSuccesses);
	testlogm("Min/Max discovery time: Min: %10lu, Max: %10lu.", test1_minTimer, test1_maxTimer);
	testlogm("Average hop count: %i hops.", test1_sumHopCounts / test1_numSuccesses);
	testlogm("Min/Max hop count: Min: %i, Max: %i.", test1_minHopCount, test1_maxHopCount);
	testfinish(TEST1_ID, TEST1_NAME);

	// Enable regular logging...
	bDontLog = false;
}

void test1_testSuccess(void) {
	test1_numSuccesses++;
	while (Timers_remove((fp_timer) test1_testTimeout, (void*) 0xFFFF));
	/* Rerun the test */
	test1_restartTest();

}
void test1_testFailure(void) {
	while (Timers_remove((fp_timer) test1_testTimeout, (void*) 0xFFFF));
	/* Rerun the test */
	test1_restartTest();
}
void test1_testTimeout(void) {
	testlogm("Test timed out.");
	testend();
	test1_restartTest();
}

void test1_restartTest(void) {
	if (bTest1_finished) {
		return;
	}
	Timers_add(TEST1_INTERVAL, (fp_timer) test1_runTest, (void*) 0xFFFF);
}

void onTest1RREPReceived(aodv_RREP_message_t *aRREP) {
	if (bTest1_finished) {
		aodv_interface_removeRREPReceivedListener(onTest1RREPReceived);
		return;
	}
	uint32_t time;
	if (aRREP->destIPAddress == test1_destination) {
		time = stopTimer(timer_test1);
		testlog(TEST1_NAME, "RREP received:");
		testlogm("Destination: %i", aRREP->destIPAddress);
		testlogm("Measured time: %10lu ms.", time);
		testlogm("Number of hops: %i.", aRREP->hopCount);
		testend();

		test1_sumTimers += time;
		if (time > test1_maxTimer) test1_maxTimer = time;
		if (time < test1_minTimer) test1_minTimer = time;

		test1_sumHopCounts += aRREP->hopCount;
		if (aRREP->hopCount > test1_maxHopCount) test1_maxHopCount = aRREP->hopCount;
		if (aRREP->hopCount < test1_minHopCount) test1_minHopCount = aRREP->hopCount;

		test1_testSuccess();
	}
}
void onPacketSent(enum aodv_sendPacketStatus_e sendStatus) {
	if (bTest1_finished) {
		return;
	}
	if (sendStatus == AODV_DESTINATION_UNREACHABLE) {
		testlog(TEST1_NAME, "AODV_DESTINATION_UNREACHABLE");
		testlogm("Test FAILED.");
		testend();
		stopTimer(timer_test1);
		test1_testFailure();
	}
}
void test1_runTest(void) {
	aodv_routingTableEntry_t *existingRoute;

	if (bTest1_finished) {
		while (Timers_remove((fp_timer) test1_runTest, (void*) 0xFFFF));
		return;
	}

	if (test1_numTestsPassed >= test1_numberOfTests) {
		bTest1_finished = true;
		while (Timers_remove((fp_timer) test1_testTimeout, (void*) 0xFFFF));
		while (Timers_remove((fp_timer) test1_runTest, (void*) 0xFFFF));
		test1_finishTest();
	}
	else {
		test1_numTestsPassed++;
		existingRoute = aodv_routingTable_lookupRoute(test1_destination);
		if (existingRoute != NULL) {
			testlogm("Deleting existing route.");
			list_deleteEntry(&ROUTING_TABLE, existingRoute);
		}

		testlogm("Sending Data to destination %i and waiting for RREP...", test1_destination);
		timer_test1 = startTimer();
		Timers_add(TEST1_TIMEOUT, (fp_timer) test1_testTimeout, (void*) 0xFFFF);
		aodv_interface_sendData(test1_destination, "DATA", 4, flags, AODV_TEST_APPLICATION, onPacketSent);
	}
}

COMMAND(TEST1_ID, CMDFLAG_SERIAL, cmdargs) {
	uint16_t count = Cmd_SplitArgs(cmdargs->args);
	const char *args_data = cmdargs->args;
	bool bRegistered;

	if (count != 2) {
		printf("[%s]: Wrong number of parameters passed!\r\n", STRING(TEST1_ID));
		printf("        Input format: <#Testruns> <Destination>\r\n");
		return;
	}
	bRegistered = aodv_interface_addRREPReceivedListener(onTest1RREPReceived);

	test1_numberOfTests = String_parseUint16(args_data, NULL);

	args_data = (char*) Cmd_GetNextArg(args_data);

	test1_destination = String_parseUint16(args_data, NULL);

	teststart(TEST1_ID, TEST1_NAME);
	testlogm("Sending %i RREPs to destination %i.", test1_numberOfTests, test1_destination);
	testend();

	test1_initTest();
	test1_runTest();

}
/*****************************************************************************************
 *                                 END_OF_TEST 1                                         *
 *****************************************************************************************/









/*****************************************************************************************
 *           TEST 2: Throughput / 1-HOP                                                *
 *****************************************************************************************/
#define TEST2_NAME "Throughput Test"
#define TEST2_ID TEST_2_A
#define TEST2_RESULT TEST_2_A_RESULT
uint8_t test2_destination;
uint8_t test2_numberOfTests;
uint8_t test2_numTestsPassed = 0;
uint8_t test2_payloadSize;
uint8_t test2_packetsReceived = 0;
uint16_t test2_interval;
char *test2_payload;

bool bTest2_finished = false;
void test2_initTest(void);
void test2_runTest(void);
void test2_finishTest(void);
void test2_packetReceived(void);

void test2_initTest(void) {
	test2_numTestsPassed = 0;
	bTest2_finished = false;
	test2_payload = malloc(test2_payloadSize);
	test2_payload[0] = TEST_2A;
	tests_initRandom();
}

void test2_finishTest(void) {
	testlogm("Test finished. %i Packets of size %i have been sent.", test2_numberOfTests, test2_payloadSize);
	free(test2_payload);
	test2_payload = malloc(30);
	sprintf(test2_payload, "@%i %s", test2_destination, STRING(TEST2_RESULT));
	_Cmd_AsciiSerialRequest(test2_payload);
	free(test2_payload);
}
void test2_runTest(void) {
	uint16_t interval;
	if (bTest2_finished) {
		while (Timers_remove((fp_timer) test2_runTest, (void*) 0xFFFF));
		return;
	}

//	testlogm("Sending packet #%i of %i byte(s) to destination %i.", test2_numTestsPassed, test2_payloadSize, test2_destination);

	sendPlainData(test2_destination, test2_payload, test2_payloadSize);
	blink(5); // 5ms

	if (++test2_numTestsPassed < test2_numberOfTests) {
		if (test2_interval == 0) {
			interval = 5 + rand_n(95);
//			logm("    (next packet in %i ms)", interval);
		}
		else {
			interval = test2_interval;
		}
		Timers_add(interval, (fp_timer) test2_runTest, (void*) 0xFFFF);
	}
	else {
		bTest2_finished = true;
		test2_finishTest();
	}
}
void test2_packetReceived(void) {
	blinkNTimes(2);		// 2 times
	test2_packetsReceived++;
}

COMMAND(TEST2_RESULT, CMDFLAG_NONE, cmdargs) {
	teststart(TEST2_ID, TEST2_NAME);
	testlogm("#Packets arrived: %i.", test2_packetsReceived);
	testend();
	// Reset the test.
	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "#Packets: %i", test2_packetsReceived);
	test2_packetsReceived = 0;
}

COMMAND(TEST2_ID, CMDFLAG_SERIAL, cmdargs) {
	uint16_t count = Cmd_SplitArgs(cmdargs->args);
	const char *args_data = cmdargs->args;

	if (count != 4) {
		printf("[%s]: Wrong number of parameters passed!\r\n", STRING(TEST2_ID));
		printf("        Input format: <#Testruns> <delay (ms)> <PayloadSize_bytes> <Destination>\r\n");
		return;
	}

	test2_numberOfTests = String_parseUint16(args_data, NULL);

	args_data = (char*) Cmd_GetNextArg(args_data);

	test2_interval = String_parseUint16(args_data, NULL);
	if (test2_interval == 0) {
		printf("Using random intervals...\r\n");
	}

	args_data = (char*) Cmd_GetNextArg(args_data);

	test2_payloadSize = String_parseUint16(args_data, NULL);

	args_data = (char*) Cmd_GetNextArg(args_data);

	test2_destination = String_parseUint16(args_data, NULL);

	teststart(TEST2_ID, TEST2_NAME);
	testlogm("Sending %i packets of %i byte(s) to destination %i.", test2_numberOfTests, test2_payloadSize, test2_destination);
	testend();

	test2_initTest();
	test2_runTest();

}
/*****************************************************************************************
 *                                 END_OF_TEST 2                                         *
 *****************************************************************************************/


/*****************************************************************************************
 *           TEST 3: Throughput / AODV                                                   *
 *****************************************************************************************/
#define TEST3_NAME "Throughput / AODV"
#define TEST3_ID TEST_3_A
#define TEST3_RESULT TEST_3_A_RESULT
uint8_t test3_destination;
uint8_t test3_numberOfTests;
uint8_t test3_numTestsPassed = 0;
uint8_t test3_payloadSize;
uint8_t test3_packetsReceived = 0;
uint16_t test3_hopCountsReceived = 0;
uint8_t test3_packetsSent = 0;
uint16_t test3_interval;
char *test3_payload;

bool bTest3_finished = false;
void test3_initTest(void);
void test3_preparePayload(char **payload);
void test3_runTest(void);
void test3_finishTest(void);
void test3_packetSent(enum aodv_sendPacketStatus_e sendStatus);
void test3_packetReceived(const aodv_test_packet_t *testPacket);

void test3_initTest(void) {
	test3_numTestsPassed = 0;
	test3_packetsReceived = 0;
	test3_hopCountsReceived = 0;
	test3_packetsSent = 0;
	bTest2_finished = false;

	tests_initRandom();
	test3_preparePayload(&test3_payload);
}
void test3_preparePayload(char **payload) {
	aodv_test_packet_t *testPacket;
	uint8_t i;
	*payload = malloc(test3_payloadSize);
	for (i = 0; i < test3_payloadSize; i++) {
		(*payload)[i] = rand();
	}
	testPacket = (aodv_test_packet_t*) *payload;
	testPacket->type = TEST_3A;
	testPacket->forward_hopCount = 0;
	testPacket->reverse_hopCount = 0;
	testPacket->isForwardDirection = true;
}

void test3_finishTest(void) {
	testlogm("Test finished. %i Packets of size %i have been sent.", test3_numberOfTests, test3_payloadSize);
	free(test3_payload);
//	test3_payload = malloc(30);
//	sprintf(test3_payload, "@%i %s", test3_destination, STRING(TEST3_RESULT));
//	_Cmd_AsciiSerialRequest(test3_payload);
//	free(test3_payload);
}
void test3_packetReceived(const aodv_test_packet_t *testPacket) {
	blinkNTimes(2);		// 2 times
	test3_packetsReceived++;
	test3_hopCountsReceived += testPacket->forward_hopCount;
}
void test3_packetSent(enum aodv_sendPacketStatus_e sendStatus) {
	if (sendStatus == AODV_PACKET_SENT) {
		test3_packetsSent++;
		blink(5); // 5ms
	}
	else {
		blink(500);
	}
}
void test3_runTest(void) {
	uint16_t interval;
	if (bTest2_finished) {
		while (Timers_remove((fp_timer) test3_runTest, (void*) 0xFFFF));
		return;
	}

//	testlogm("Sending packet #%i of %i byte(s) to destination %i.", test3_numTestsPassed, test3_payloadSize, test3_destination);

	aodv_interface_sendData(test3_destination, test3_payload, test3_payloadSize, AODV_FLAG_NONE, AODV_TEST_APPLICATION, test3_packetSent);

	if (++test3_numTestsPassed < test3_numberOfTests) {
		if (test3_interval == 0) {
			interval = 5 + rand_n(95);
//			logm("    (next packet in %i ms)", interval);
		}
		else {
			interval = test3_interval;
		}
		Timers_add(interval, (fp_timer) test3_runTest, (void*) 0xFFFF);
	}
	else {
		bTest2_finished = true;
		test3_finishTest();
	}
}

COMMAND(TEST3_RESULT, CMDFLAG_NONE, cmdargs) {
	teststart(TEST3_ID, TEST3_NAME);
	testlogm("#Packets: %i, Hops: %i", test3_packetsReceived, test3_hopCountsReceived / test3_packetsReceived);
	testend();
	// Reset the test.

	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "#Packets: %i, Hops: %i", test3_packetsReceived, test3_hopCountsReceived / test3_packetsReceived);

	test3_packetsReceived = 0;
	test3_hopCountsReceived = 0;
}

COMMAND(TEST3_ID, CMDFLAG_SERIAL, cmdargs) {
	uint16_t count = Cmd_SplitArgs(cmdargs->args);
	const char *args_data = cmdargs->args;

	if (count != 4) {
		printf("[%s]: Wrong number of parameters passed!\r\n", STRING(TEST3_ID));
		printf("        Input format: <#Testruns> <delay (ms)> <PayloadSize_bytes> <Destination>\r\n");
		return;
	}

	test3_numberOfTests = String_parseUint16(args_data, NULL);

	args_data = (char*) Cmd_GetNextArg(args_data);

	test3_interval = String_parseUint16(args_data, NULL);
	if (test3_interval == 0) {
		printf("Using random intervals...\r\n");
	}

	args_data = (char*) Cmd_GetNextArg(args_data);

	test3_payloadSize = String_parseUint16(args_data, NULL);

	if (test3_payloadSize < sizeof(aodv_test_packet_t)) {
		printf("[%s]: Payload size must be at least %i bytes!\r\n", STRING(TEST3_ID), sizeof(aodv_test_packet_t));
		printf("        Input format: <#Testruns> <delay (ms)> <PayloadSize_bytes> <Destination>\r\n");
		return;
	}

	args_data = (char*) Cmd_GetNextArg(args_data);

	test3_destination = String_parseUint16(args_data, NULL);

	teststart(TEST3_ID, TEST3_NAME);
	testlogm("Sending %i packets of %i byte(s) to destination %i.", test3_numberOfTests, test3_payloadSize, test3_destination);
	testend();

	test3_initTest();
	test3_runTest();

}
/*****************************************************************************************
 *                                 END_OF_TEST 3                                         *
 *****************************************************************************************/



void processTestPacket(const void *data, const uint16_t size) {
	if (size < 1) {
		return;
	}
	switch (((uint8_t*) data)[0]) {
	case TEST_1A : {
		break;
	}
	case TEST_2A : {
		test2_packetReceived();
		break;
	}
	}
}

void processTestAODVPacket(aodv_dataPacket_t *dataPacket) {
	aodv_test_packet_t *testPacket;
	if (dataPacket->header.application != AODV_TEST_APPLICATION) {
		return;
	}
	testPacket = (aodv_test_packet_t*) dataPacket->payload;
	if (testPacket->isForwardDirection) {
		testPacket->forward_hopCount++;
	}
	else {
		testPacket->reverse_hopCount++;
	}
	switch (testPacket->type) {
	case TEST_2A : {
		break;
	}
	case TEST_3A : {
		test3_packetReceived(testPacket);
		break;
	}
	default: break;
	}

}






