/*
 * trafficGenerator.c
 *
 *  Created on: 30.04.2009
 *      Author: Janos
 */

#include <string.h>

#include "controller.trafficGenerator.h"

void sendTraficGeneratorPacket(uint8_t destID);
void generatePacket(void* arg);

#define MAX_PAYLOAD_SIZE 	(256)
bool genStarted = false;
uint8_t genDestID = 255;
uint16_t genFrequency;
uint16_t genNumPackets = 0;
uint16_t genPayloadSize = 0;
char genPayloadBuffer[MAX_PAYLOAD_SIZE];

bool dumpStarted = false;
bool dumpPromMode = false;
bool dumpVerboseMode = false;

void sendTraficGeneratorPacket(uint8_t destID) {
	netpacket_send_args_t npsargs;
	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = destID;
	npsargs.netheader.type = TRAFFIC_GENERATOR_PACKET;
	npsargs.netheader.flags = 0;
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = genPayloadBuffer;
	npsargs.payload[0].size = genPayloadSize;

	Net_send(&npsargs);

	blink(10);
}

void dumpPacket(struct netpacket_handler_args* dumpPacket, bool addrThisNode) {
	uint8_t i;
	char *payload;

	if (!dumpStarted)
		return;

	if (!addrThisNode && !dumpPromMode) {
		/* Keine Behandlung von Dump-Paketen, die nicht an diesen Knoten
		 * gerichtet sind, wenn 'PROM_MODE' nicht aktiv ist.
		 */
		return;
	}
	switch (dumpVerboseMode) {
		case true: {
			printf("Dumping Packet:\r\n");
			printf("\t\t        From: \t%u\r\n", dumpPacket->netheader->from);
			printf("\t\t          To: \t%u\r\n", dumpPacket->netheader->to);
			payload = (char*) dumpPacket->payload;
			printf("\t\tPayload-Size: \t%u\r\n", dumpPacket->payload_size);
			printf("\t\t     Payload: \t");
			for (i = 0; i < dumpPacket->payload_size; i++)
				printf("%c", *payload++);
			printf("\r\n");
			break;
		}
		default: {
			printf("Dumping Packet (from %u to %u).\r\n", dumpPacket->netheader->from, dumpPacket->netheader->to);
		}
	}

}

void stopGeneration(void) {
	genStarted = false;
	printf("TrafficGenerator stopped.\r\n");
}

void generatePacket(void* arg) {
	arg = 0;
	if (!genStarted) { /* Schleife beenden, wenn 'genStarted' FALSE ist. */
		return;
	}

	sendTraficGeneratorPacket(genDestID);
	printf("Paket %u generiert\r\n", genNumPackets);

	if (--genNumPackets) { /* Schleife nur fortsetzen, wenn 'genAmount' noch >0 ist. */
		Timers_add(genFrequency, generatePacket, (void*) 0xFFFF);
	} else {
		printf("TrafficGenerator finished.\r\n");
		genStarted = false;
	}
}

void startGeneration(uint8_t destID, uint16_t freq, uint16_t numPackets,
		uint16_t payloadSize) {
	genDestID = destID;
	genFrequency = freq;
	genNumPackets = numPackets;
	genPayloadSize = payloadSize;

	printf("TrafficGenerator started. Sending %u packeges every %u ms.\r\n",
			numPackets, freq);
	printf("Type 'genPackets stop' to stop this generation.\r\n");

	memset(genPayloadBuffer, 'X', genPayloadSize);

	if (genNumPackets && !genStarted) { /* Nur neu starten, wenn der TG noch nicht gestartet ist.
	 * Ansonsten brnimmt der laufende TG die generten Werte. */
		genStarted = true;
		Timers_add(genFrequency, generatePacket, (void*) 0xFFFF);
	}

}

void startDumping(bool promiscuousMode, bool verboseMode) {
	dumpPromMode = promiscuousMode;
	dumpVerboseMode = verboseMode;
	dumpStarted = true;
	printf("Dumping has been started%s.\r\n",
			(promiscuousMode ? " in PROMISCUOUS mode" : ""));
}

void stopDumping(void) {
	dumpStarted = false;
	printf("Dumping has been stopped.\r\n");
}

/*
 * \brief Startet oder beendet den TrafficGenerator.
 * Startet oder beendet den TrafficGenerator entsprechend
 * der eingegebenen Parameter. Dabei ist folgende Syntax
 * einzuhalten:
 *
 * genPackets <action>[ <dest> <freq> <numPackets> <payloadSize>]
 * 		<action>:		start|STAR|stop|STOP
 * 			start, START:		TrafficGenerator starten.
 * 			stop, STOP:			TrafficGenerator stoppen.
 * 		<dest>:			Ziel-Adresse. (Knoten-ID).
 * 		<freq>:			Frequenz in Millisekunden.
 * 		<numPackets>:	Anzahl der gesendeten Pakete.
 * 		<payloadSize>:	Gre der gesendeten Nutzlast (in bytes).
 *
 * Die Parameter <dest>, <freq>, <numPackets> und <payloadSize> sind obligatorisch,
 * wenn <action>=start bergeben wurde.
 */
COMMAND(genPackets, CMDFLAG_SERIAL, cmdargs)
{
	uint16_t freq;
	uint16_t numPackets;
	uint16_t payloadSize;
	uint8_t destID;
	char* arg;
	uint16_t count = Cmd_SplitArgs(cmdargs->args);

	if (count < 1) {
		printf("Wrong number of parameters passed. Input format is:\r\n");
		printf("\t\tgenPackets {stop|STOP}\r\n");
		printf(
				"\t\tgenPackets {start|START} <dest> <freq> <numPackets> <payloadSize>\r\n");
		return;
	}

	arg = (char*) cmdargs->args;

	if (!strcmp(arg, "stop") || !strcmp(arg, "STOP")) {
		stopGeneration();
		return;
	}
	if (!strcmp(arg, "start") || !strcmp(arg, "START")) {
		if (count != 5) {
			printf("Wrong number of parameters passed! Expected:\r\n");
			printf(
					"\t\tgenPackets {start|START} <dest> <freq> <numPackets> <payloadSize>\r\n");
			return;
		}

		arg = (char*) Cmd_GetNextArg(arg);
		destID = String_parseUint16(arg, NULL);
		arg = (char*) Cmd_GetNextArg(arg);
		freq = String_parseUint16(arg, NULL);
		arg = (char*) Cmd_GetNextArg(arg);
		numPackets = String_parseUint16(arg, NULL);
		arg = (char*) Cmd_GetNextArg(arg);
		payloadSize = String_parseUint16(arg, NULL);
		if (payloadSize > MAX_PAYLOAD_SIZE)
			payloadSize = MAX_PAYLOAD_SIZE;

		startGeneration(destID, freq, numPackets, payloadSize);

		return;
	}
	printf("Unknown parameter: %s\r\n", arg);
}

/*
 * \brief Started oder beendet das Dumping von Paketen.
 * Started oder beendet das Dumping von Paketen. Pakete knnen
 * im promiscuous Modus gedumpt werden - darin werden auch
 * Pakete empfangen, die nicht an diesen Knoten gesendet wurden.
 *
 * Syntax:
 * dump <action>[ <promiscuous>] [ <verbose>]
 * 		<action>:		start|START|stop|STOP
 * 			start, START:			Dumping starten.
 * 			stop, STOP:				Dumping beenden.
 * 		<promiscuous>:	prom|PROM
 * 			prom, PROM:				Promiscuous Modus.
 * 		<verbose>:		verbose|VERBOSE
 * 			verbose, VERBOSE:		Details zu empfangenen Paketen.
 *
 * Die Parameter <promiscuous> und <verbose> sind optional, aber nur erlaubt,
 * wenn <action>=start ist.
 */
COMMAND(dump, CMDFLAG_SERIAL, cmdargs)
{
	char* arg;
	uint16_t count = Cmd_SplitArgs(cmdargs->args);

	bool prom = false, verbose = false;

	if (count < 1) {
		printf("Wrong number of parameters passed! (expected at least one)\r\n");
		return;
	}

	arg = (char*) cmdargs->args;

	if (!strcmp(arg, "stop") || !strcmp(arg, "STOP")) {
		stopDumping();
		return;
	}
	if (!strcmp(arg, "start") || !strcmp(arg, "START")) {
		if (count > 1) {
			arg = (char*) Cmd_GetNextArg(arg);
			if (!strcmp(arg, "prom") || !strcmp(arg, "PROM")) {
				prom = true;
			} else if (!strcmp(arg, "verbose") || !strcmp(arg, "VERBOSE")) {
				verbose = true;
			} else {
				printf("Invalid parameter #%i: '%s'.\r\n", 2, arg);
				return;
			}
		}
		if (count > 2) {
			arg = (char*) Cmd_GetNextArg(arg);
			if (!strcmp(arg, "verbose") || !strcmp(arg, "VERBOSE")) {
				verbose = true;
			} else {
				printf("Invalid parameter #%i: '%s'.\r\n", 3, arg);
				return;
			}
		}
		startDumping(prom, verbose);
		return;
	}
	printf("Invalid parameter #%i: '%s'.\r\n", 1, arg);
}
